package org.sevensource.commons.configuration.jdbc.database;

import org.apache.commons.configuration.CompositeConfiguration;
import org.apache.commons.configuration.Configuration;
import org.apache.commons.lang.NotImplementedException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ExtendedCompositeConfiguration extends CompositeConfiguration {

	private final static Logger logger = LoggerFactory.getLogger(ExtendedCompositeConfiguration.class);
	
	private final String defaultsIdentifier;
	
	public ExtendedCompositeConfiguration(String defaultsIdentifier) {
		this.defaultsIdentifier = defaultsIdentifier;
	}
	
	public String getDefaultsIdentifier() {
		return defaultsIdentifier;
	}
	
	@Override
	public void addProperty(String key, Object value) {
		if (logger.isWarnEnabled()) {
			logger.warn("Adding property with [{}]=[{}] to IN-MEMORY configuration - NOT PERSISTENT!", key, value);
		}
		
		super.addProperty(key, value);
	}
	
	@Override
	public void setProperty(String key, Object value) {
		logger.error("Adding properties on the *CompositeConfiguration* is disabled, since it also clears the values of the underlying database configurations");
		throw new NotImplementedException("Adding properties is disabled, since it also clears the values of the underlying database configurations");
	}
	
	public void clearCache() {
		for(int i=0; i<getNumberOfConfigurations();i++) {
			Configuration config = getConfiguration(i);
			if(config instanceof CachingDatabaseConfiguration) {
				if (logger.isDebugEnabled()) {
					logger.debug("Clearing cache on config {}", config);
				}
				((CachingDatabaseConfiguration) config).clearCache();
			}
		}
	}
}
