package org.sevensource.commons.configuration.jdbc.database;

import javax.sql.DataSource;

import org.apache.commons.configuration.Configuration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class ExtendedDatabaseConfigurationFactory {
	
	private final static Logger logger = LoggerFactory.getLogger(ExtendedDatabaseConfigurationFactory.class);
	
	
	private String table;
	private String nameColumn;
	private String keyColumn;
	private String valueColumn;
	private String defaultsIdentifier;
	
	private final DataSource dataSource;
	private final String serverName;
	private final String applicationName;
	
	public ExtendedDatabaseConfigurationFactory(DataSource dataSource, String serverName, String applicationName) {
		this.dataSource = dataSource;
		this.serverName = serverName;
		this.applicationName = applicationName;
	}
	
	public ExtendedCompositeConfiguration buildConfiguration() {
		
		Configuration serverConfiguration = buildDatabaseConfiguration(getServerName());
		Configuration defaultConfiguration = buildDatabaseConfiguration(getDefaultsIdentifier());
		
		ExtendedCompositeConfiguration config = new ExtendedCompositeConfiguration( getDefaultsIdentifier() );
		//addListeners(config);
		
		config.addConfiguration(serverConfiguration );
		config.addConfiguration(defaultConfiguration );
		
		return config;
	}
	
	private Configuration buildDatabaseConfiguration(String subset) {
		
		if (logger.isDebugEnabled()) {
			logger.debug("Building databaseConfiguration for subset {}", subset);
		}
		
		CachingDatabaseConfiguration config = new CachingDatabaseConfiguration(
				getDataSource(), getTable(), getNameColumn(), 
				getKeyColumn(), getValueColumn(), getApplicationName(),
				false);
		
		//addListeners(config);
		return config.subset( subset );
	}
	
	/*
	private void addListeners(EventSource config) {
		ConfigurationListener listener = new ExtendedDatabaseConfigurationListener();
		config.addConfigurationListener(listener);
		config.addErrorListener((ConfigurationErrorListener) listener);
	}
	*/
	
	
	private DataSource getDataSource() {
		return dataSource;
	}
	
	
	public String getTable() {
		return table;
	}
	public void setTable(String table) {
		this.table = table;
	}
	public String getNameColumn() {
		return nameColumn;
	}
	public void setNameColumn(String nameColumn) {
		this.nameColumn = nameColumn;
	}
	public String getKeyColumn() {
		return keyColumn;
	}
	public void setKeyColumn(String keyColumn) {
		this.keyColumn = keyColumn;
	}
	public String getValueColumn() {
		return valueColumn;
	}
	public void setValueColumn(String valueColumn) {
		this.valueColumn = valueColumn;
	}
	public String getDefaultsIdentifier() {
		return defaultsIdentifier;
	}
	public void setDefaultsIdentifier(String defaultsIdentifier) {
		this.defaultsIdentifier = defaultsIdentifier;
	}
	public String getServerName() {
		return serverName;
	}
	public String getApplicationName() {
		return applicationName;
	}
}
