package org.sevensource.commons.configuration.jdbc.database;

import org.apache.commons.configuration.AbstractConfiguration;
import org.apache.commons.configuration.event.ConfigurationErrorEvent;
import org.apache.commons.configuration.event.ConfigurationErrorListener;
import org.apache.commons.configuration.event.ConfigurationEvent;
import org.apache.commons.configuration.event.ConfigurationListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ExtendedDatabaseConfigurationListener implements ConfigurationListener, ConfigurationErrorListener {
	
	private final static Logger logger = LoggerFactory.getLogger(ExtendedDatabaseConfigurationListener.class);
	
	public void configurationError(ConfigurationErrorEvent event) {
		logger.error("Configuration error for property [{}] with eventType [{}].", event.getPropertyName(), event.getType());
	}

	
	public void configurationChanged(ConfigurationEvent event) {
		switch (event.getType()) {
		case AbstractConfiguration.EVENT_ADD_PROPERTY:
			if(! event.isBeforeUpdate()) {
				if (logger.isInfoEnabled()) {
					logger.info("Property [{}]=[{}] added to configuration.", event.getPropertyName(), event.getPropertyValue());
				}
			}
			break;
			
		case AbstractConfiguration.EVENT_SET_PROPERTY:
			if(! event.isBeforeUpdate()) {
				if (logger.isInfoEnabled()) {
					logger.info("Property [{}] set to new value [{}].", event.getPropertyName(), event.getPropertyValue());
				}
			} else {
				if (logger.isDebugEnabled()) {
					logger.debug("Going to set property [{}] to new value [{}].", event.getPropertyName(), event.getPropertyValue());
				}
			}
			break;
			
		case AbstractConfiguration.EVENT_CLEAR:
			if(! event.isBeforeUpdate()) {
				if (logger.isInfoEnabled()) {
					logger.info("Configuration has been cleared");
				}
			}
			break;
			
		case AbstractConfiguration.EVENT_CLEAR_PROPERTY:
			if(! event.isBeforeUpdate()) {
				if (logger.isInfoEnabled()) {
					logger.info("Property [{}] has been cleared", event.getPropertyName());
				}
			}
			break;
			
		case AbstractConfiguration.EVENT_READ_PROPERTY:
			if (logger.isDebugEnabled()) {
				logger.debug("Property [{}] with value [{}] has been read.", event.getPropertyName(), event.getPropertyValue());
			}
			break;
			
		default:
			if (logger.isWarnEnabled()) {
				logger.warn("Unknown ConfigurationChange Event with type-id [{}]", event.getType());
			}
			break;
		}
	}
}
